@@ -3,9 +3,8 @@ name = "dataset_iterator"
 from .index_array_iterator import IndexArrayIterator
 from .multichannel_iterator import MultiChannelIterator
 from .tracking_iterator import TrackingIterator
-from .dy_iterator import DyIterator
-from .delta_iterator import DeltaIterator
-from .tile_utils import extract_tiles, augment_tiles, extract_tile_function, augment_tiles_inplace
+from .tile_utils import extract_tiles, augment_tiles, extract_tile_function, extract_tile_random_zoom_function, augment_tiles_inplace
 from .preprocessing_image_generator import PreProcessingImageGenerator
 
 from .datasetIO import DatasetIO, H5pyIO, MultipleFileIO, MultipleDatasetIO, ConcatenateDatasetIO, MemoryIO
+from .utils import enrich_with_hardest_indices
