@@ -7,9 +7,13 @@ import time
 import copy
 from math import copysign, ceil
 from .datasetIO import DatasetIO, get_datasetIO, MemoryIO
-from .utils import ensure_multiplicity, flatten_list
+from .utils import ensure_multiplicity, flatten_list, replace_last
 from itertools import groupby
 from .index_array_iterator import IMCOMPLETE_LAST_BATCH_MODE
+try:
+    import elasticdeform as ed
+except ImportError:
+    ed = None
 
 class MultiChannelIterator(IndexArrayIterator):
     """Flexible Image iterator allowing on-the-fly pre-processing / augmentation of data of massive multichannel datasets.
@@ -42,7 +46,8 @@ class MultiChannelIterator(IndexArrayIterator):
         input is a dict mapping channel index to batch
         applied after image_data_generators and before weight_map_functions / output_postprocessing_functions if any.
     extract_tile_function : callable
-        function that inputs a batch, splits it into tiles and concatenate along batch axis.
+        function that inputs a batch and a bool (that is true if the batch is a mask), splits it into tiles and concatenate along batch axis.
+        when the iterator has several channels, the function must handle a list of batches (and a list of bool) to tile them simultaneously.
         Applied before channels_postprocessing_function and after image_data_generators
     mask_channels : list of ints
         index of channels that contain binary (or labeled) images. Used to detect the presence/abscence of segmented objects at image borders
@@ -50,18 +55,20 @@ class MultiChannelIterator(IndexArrayIterator):
         output = (output) * output_multiplicity
     input_multiplicity : int
         input = (input) * input_multiplicity
-    channel_scaling_param : list of dict
-        channel scaling parameter: each element should be as: {'level':1, 'qmin':5, 'qmax':95}, where:
-        qmin is the min percentile, qmax the max percentile, Level represent the hierachical level at which computing the percentile.
-        scaling is: I -> ( I - qmin ) / ( qmax - qmin )
     group_keyword : string or list of strings
-        string contained in the path of all channels -> allows to limit iteration to a subset of the dataset.
+        string contained in the path of all channels -> allows to limit iteration to a subset of the dataset. If dataset is h5: group_keywords can contain .* regular expression
+    group_scaling: list of list
+        group-wise+channel-wise scaling
+        should be of same length as group number, each sub-list should be either None either a list of same length as channel number.
+        each element should be either None, a callable, that is called on a single image and return the scaled image, either [center, scale] (scaled image will be I -> ( I - center) / scale )
     image_data_generators : list of image ImageDataGenerator as defined in keras pre-processing, for data augmentation.
         should be of same size as channel_keywords.
         augmentation parameters are computed on the first channel, and applied on each channels using the ImageDataGenerator of corresponding index.
         if a mask_channels is not empty, then the first mask channel is used as reference for parameters computation instead of the first channel.
     singleton_channels : list of ints
         list of channel that contains a single image.
+    channel_slicing_channels : None or dictionary
+        dict that map channel index to a slice (or a callable that inputs the number of channels and returns a slice), that will be applied to the last axis of the corresponding dataset
     n_spatial_dims : int (2 or 3)
         number of spatial dimensions. if 2, 4D tensor are returned, if 3 5D tensor are returned.
     batch_size : int
@@ -70,6 +77,12 @@ class MultiChannelIterator(IndexArrayIterator):
         if true, image indices are shuffled
     perform_data_augmentation : boolean
         if false, image_data_generators are ignored
+    elasticdeform_parameters : dict
+        parameters passed to elasticdeform function. see: https://github.com/gvtulder/elasticdeform/blob/master/elasticdeform/deform_grid.py
+        main parameters are: "sigma" and "points". alternatively "grid_spacing" can be passed and points will be infered (size//grid_spacing) as well as sigma (min(size/points)).
+        "axis" should not be provided. default "order" is 1 and automatically set to 0 for mask channels.
+        mode: out-of-bound strategy
+        Applied before channels_postprocessing_function and extract_tile and after image_data_generators
     seed : int
         random seed
     dtype : numpy datatype
@@ -115,7 +128,6 @@ class MultiChannelIterator(IndexArrayIterator):
     n_spatial_dims
     group_keyword
     channel_keywords
-    channel_scaling_param
     dtype
     perform_data_augmentation
     singleton_channels
@@ -143,26 +155,45 @@ class MultiChannelIterator(IndexArrayIterator):
                 mask_channels=[],
                 output_multiplicity = 1,
                 input_multiplicity = 1,
-                channel_scaling_param=None,
                 group_keyword=None,
+                group_proportion=None,
+                group_scaling=None,
                 image_data_generators=None,
                 singleton_channels=[],
+                channel_slicing_channels=None,
                 n_spatial_dims=2,
                 batch_size=32,
                 shuffle=True,
                 perform_data_augmentation=True,
+                elasticdeform_parameters=None,
                 seed=None,
                 dtype='float32',
+                convert_masks_to_dtype=True,
                 memory_persistant=False,
                 incomplete_last_batch_mode=IMCOMPLETE_LAST_BATCH_MODE[0]):
         self.dataset = dataset
         self.memory_persistant=memory_persistant
         self.n_spatial_dims=n_spatial_dims
         self.group_keyword=group_keyword
+        self.group_proportion=group_proportion
+        self.group_scaling = group_scaling
+        self.group_proportion_init=False
+        if group_proportion is not None:
+            assert group_keyword is not None and isinstance(group_keyword, (tuple, list)) and isinstance(group_proportion, (tuple, list)) and len(group_proportion)==len(group_keyword), "when group_proportion is not None, group_keyword should be a list/tuple group_proportion should be of same length as group_keyword"
+        if self.group_scaling is not None:
+            n_grps = len(group_keyword) if isinstance(group_keyword, (tuple, list)) else 1
+            assert isinstance(group_scaling, (tuple, list)) and len(group_scaling) == n_grps, "group_scaling is not None, and should be of same length as group number"
+            for grp_idx, grp in enumerate(group_scaling):
+                assert grp is None or (isinstance(grp, (tuple, list)) and len(grp) == len(channel_keywords)), "scaling parameters for group {} should be either None either of same length as channel numbers".format(grp_idx)
         self.channel_keywords=channel_keywords
-        self.channel_scaling_param = channel_scaling_param
         self.dtype = dtype
+        self.convert_masks_to_dtype=convert_masks_to_dtype
         self.perform_data_augmentation=perform_data_augmentation
+        self.channel_slicing_channels = channel_slicing_channels if channel_slicing_channels is not None else {}
+        if elasticdeform_parameters is not None:
+            assert isinstance(elasticdeform_parameters, dict)
+            assert ed is not None, "elasticdeform package is not installed but parameters are specified"
+        self.elasticdeform_parameters=elasticdeform_parameters
         self.singleton_channels=[] if singleton_channels is None else singleton_channels
         assert isinstance(self.singleton_channels, list), "singleton_channels must be a list"
         if len(self.singleton_channels)>0:
@@ -237,7 +268,8 @@ class MultiChannelIterator(IndexArrayIterator):
                         self.batch_size=1
         # labels
         try:
-            self.labels = [self.datasetIO.get_dataset(path.replace(self.channel_keywords[0], '/labels')) for path in self.paths]
+            label_path = [replace_last(path, self.channel_keywords[0], '/labels') for path in self.paths]
+            self.labels = [self.datasetIO.get_dataset(path) if path in self.datasetIO else None for path in label_path]
             for i, ds in enumerate(self.labels):
                 self.labels[i] = np.char.asarray(ds[()].astype('unicode')) # todo: check if necessary to convert to char array ? unicode is necessary
             if len(self.labels)!=len(self.ds_array[0]):
@@ -246,28 +278,7 @@ class MultiChannelIterator(IndexArrayIterator):
                 raise ValueError('Invalid input file: at least one dataset has element numbers that differ from corresponding label array')
         except:
             self.labels = None
-        # set scaling information for each dataset
-        self.channel_scaling = [None]*len(channel_keywords)
-        if self.channel_scaling_param!=None:
-            percentile_x = np.arange(0, 101)
-            for c, scaling_info in enumerate(self.channel_scaling_param):
-                if scaling_info is not None:
-                    self.channel_scaling[c]=[None]*len(self.paths)
-                    for ds_idx, path in enumerate(self.paths):
-                        group = self.datasetIO.get_parent_path(path)
-                        for i in range(scaling_info.get('level', 1)):
-                            group = self.datasetIO.get_parent_path(group)
-                            if group==None:
-                                raise ValueError("scaling group level too high for channel {}({}) group path: {}".format(c, channel_keywords[c]), self.datasetIO.get_parent_path(path))
-                        # percentiles are located in attributes of group
-                        # TODO test if this raise an error when not present
-                        percentiles = self.datasetIO.get_attribute(group, channel_keywords[c].replace('/', '')+'_percentiles')
-                        if percentiles is None:
-                            raise ValueError("No percentile array found in group {} for channel: {}({})".format(group, c, channel_keywords[c]))
 
-                        # get IQR and median
-                        minv, med, maxv = np.interp([scaling_info.get('qmin', 5), 50, scaling_info.get('qmax', 95)], percentile_x, percentiles)
-                        self.channel_scaling[c][ds_idx] = [med, maxv-minv]
         if not memory_persistant:
             self._close_datasetIO()
         self.void_mask_max_proportion = -1
@@ -275,7 +286,8 @@ class MultiChannelIterator(IndexArrayIterator):
             self.void_mask_chan = mask_channels[0]
         else:
             self.void_mask_chan=-1
-        super().__init__(indexes[-1], batch_size, shuffle, seed, incomplete_last_batch_mode)
+        self.total_n = indexes[-1]
+        super().__init__(self.total_n, batch_size, shuffle, seed, incomplete_last_batch_mode)
 
     def _open_datasetIO(self):
         self.datasetIO = get_datasetIO(self.dataset, 'r')
@@ -342,14 +354,13 @@ class MultiChannelIterator(IndexArrayIterator):
 
         return train_iterator, test_iterator
 
-    def _get_ds_idx(self, index_array):
+    def _get_ds_idx(self, index_array): # !! modifies index_array
         ds_idx = np.searchsorted(self.ds_len, index_array, side='right')
         index_array -= self.ds_off[ds_idx] # remove ds offset to each index
         return ds_idx
 
     def _get_grp_idx(self, index_array):
         grp_idx = np.searchsorted(self.grp_len, index_array, side='right')
-        index_array -= self.grp_off[grp_idx] # remove ds offset to each index
         return grp_idx
 
     def _get_batches_of_transformed_samples(self, index_array):
@@ -384,7 +395,7 @@ class MultiChannelIterator(IndexArrayIterator):
             output = _apply_multiplicity(output, self.output_multiplicity) # removes None batches
             return (input, output)
 
-    def _get_batch_by_channel(self, index_array, perform_augmentation, input_only=False):
+    def _get_batch_by_channel(self, index_array, perform_augmentation, input_only=False, perform_elasticdeform=True, perform_tiling=True):
         if self.datasetIO is None: # for concurency issues: file is open lazyly by each worker
             self._open_datasetIO()
         index_array = np.copy(index_array) # so that main index array is not modified
@@ -398,17 +409,45 @@ class MultiChannelIterator(IndexArrayIterator):
         for chan_idx in channels:
             batch_by_channel[chan_idx] = self._get_batches_of_transformed_samples_by_channel(index_ds, index_array, chan_idx, channels[0], aug_param_array, perform_augmentation=perform_augmentation)
 
-        if self.extract_tile_function is not None:
-            numpy_rand_state = np.random.get_state()
-            for chan_idx in channels:
-                np.random.set_state(numpy_rand_state) # ensure same tile + aug if tile fun implies randomness
-                batch_by_channel[chan_idx] = self.extract_tile_function(batch_by_channel[chan_idx])
+        if perform_elasticdeform:
+            self._apply_elasticdeform(batch_by_channel)
+        if perform_tiling:
+            self._apply_tiling(batch_by_channel)
 
         if self.channels_postprocessing_function is not None:
             self.channels_postprocessing_function(batch_by_channel)
 
         return batch_by_channel, aug_param_array, channels[0]
 
+    def _apply_elasticdeform(self, batch_by_channel):
+        if self.elasticdeform_parameters is not None:
+            channels = [c for c in batch_by_channel.keys() if c>=0]
+            order = self.elasticdeform_parameters.pop("order", 1)
+            order = ensure_multiplicity(len(channels), order)
+            if len(self.mask_channels)>0:
+                for i, chan_idx in enumerate(channels):
+                    if chan_idx in self.mask_channels:
+                        order[i]=0
+            axis = tuple([i+1 for i in range(self.n_spatial_dims)])
+            mode = self.elasticdeform_parameters.pop('mode', 'mirror')
+            image_shape = batch_by_channel[channels[0]].shape[1:-1]
+            grid_spacing = ensure_multiplicity(len(image_shape), self.elasticdeform_parameters.pop('grid_spacing', 15))
+            points = self.elasticdeform_parameters.pop('points', [max(3, s//gs) for s, gs in zip(image_shape, grid_spacing)])
+            sigma = self.elasticdeform_parameters.pop('sigma', np.min([s/p for s, p in zip(image_shape, points)]))
+            batches = [batch_by_channel[chan_idx] for chan_idx in channels]
+            batches = ed.deform_random_grid(batches, order = order, sigma=sigma, mode=mode, axis=axis, **self.elasticdeform_parameters)
+            for i, chan_idx in enumerate(channels):
+                batch_by_channel[chan_idx] = batches[i]
+
+    def _apply_tiling(self, batch_by_channel):
+        if self.extract_tile_function is not None:
+            channels = [c for c in batch_by_channel.keys() if c>=0]
+            batches = [batch_by_channel[chan_idx] for chan_idx in channels]
+            is_mask = [chan_idx in self.mask_channels for chan_idx in channels]
+            batches = self.extract_tile_function(batches, is_mask)
+            for i, chan_idx in enumerate(channels):
+                batch_by_channel[chan_idx] = batches[i]
+
     def _get_input_batch(self, batch_by_channel, ref_chan_idx, aug_param_array):
         if len(self.input_channels)==1:
             return batch_by_channel[self.input_channels[0]]
@@ -482,7 +521,9 @@ class MultiChannelIterator(IndexArrayIterator):
 
     def _read_image_batch(self, index_ds, index_array, chan_idx, ref_chan_idx, aug_param_array):
         # read all images # TODO read all image per ds at once.
-        images = [self._read_image(chan_idx, ds_idx, im_idx) for i, (ds_idx, im_idx) in enumerate(zip(index_ds, index_array))]
+        # in case of group scaling : group index is retrieved:
+        grp_idx = self._get_grp_idx(index_array + self.ds_off[index_ds]) if self.group_scaling is not None else [0]*len(index_array)
+        images = [self._read_image(chan_idx, ds_idx, im_idx, grp_idx) for i, (ds_idx, im_idx, grp_idx) in enumerate(zip(index_ds, index_array, grp_idx))]
         batch = np.stack(images)
         return batch
 
@@ -497,14 +538,19 @@ class MultiChannelIterator(IndexArrayIterator):
                 pass
         return params
 
-    def _read_image(self, chan_idx, ds_idx, im_idx):
+    def _read_image(self, chan_idx, ds_idx, im_idx, grp_idx=0):
         ds = self.ds_array[chan_idx][ds_idx]
         if chan_idx in self.singleton_channels:
             im_idx=0
         im = ds[im_idx]
         if len(im.shape)==self.n_spatial_dims: # add channel axis
             im = np.expand_dims(im, -1)
-        im = im.astype(self.dtype, copy=False) # copy
+        elif chan_idx in self.channel_slicing_channels:
+            chan_slice = self.channel_slicing_channels[chan_idx]
+            im = im[...,chan_slice(im.shape[-1])] if callable(chan_slice) else im[...,chan_slice]
+        if self.convert_masks_to_dtype or chan_idx not in self.mask_channels:
+            im = im.astype(self.dtype, copy=False)
+
         # apply dataset-wise scaling if information is present in attributes
         off = self.ds_scaling_center[chan_idx][ds_idx]
         factor = self.ds_scaling_factor[chan_idx][ds_idx]
@@ -512,24 +558,23 @@ class MultiChannelIterator(IndexArrayIterator):
             im = (im - off)/factor
 
         # apply group-wise scaling
-        off, factor = self._get_scaling(chan_idx, ds_idx)
-        if off!=0 or factor!=1:
-            im = (im - off) / factor
-        if chan_idx in self.mask_channels:
+        if self.group_scaling is not None and self.group_scaling[grp_idx] is not None and self.group_scaling[grp_idx][chan_idx] is not None:
+            scaling = self.group_scaling[grp_idx][chan_idx]
+            if callable(scaling):
+                im = scaling(im)
+            else:
+                im = ( im - scaling[0] ) / scaling[1]
+
+        # in case of lossy compression: mask must be 0 outside
+        if chan_idx in self.mask_channels and not issubclass(im.dtype.type, np.integer):
             im[np.abs(im) < 1e-10] = 0
         return im
 
-    def _get_scaling(self, chan_idx, ds_idx):
-        if self.channel_scaling==None or self.channel_scaling[chan_idx]==None:
-            return (0, 1)
-        else:
-            return self.channel_scaling[chan_idx][ds_idx]
-
     def _get_dataset_path(self, channel_idx, ds_idx):
         if channel_idx==0:
             return self.paths[ds_idx]
         else:
-            return self.paths[ds_idx].replace(self.channel_keywords[0], self.channel_keywords[channel_idx])
+            return replace_last(self.paths[ds_idx], self.channel_keywords[0], self.channel_keywords[channel_idx])
 
     def inspect_indices(self, index_array):
         a = np.array(index_array, dtype=np.int)
@@ -567,7 +612,7 @@ class MultiChannelIterator(IndexArrayIterator):
 
         for ds_i, ds_i_i, ds_i_len in zip(*np.unique(self._get_ds_idx(self.index_array), return_index=True, return_counts=True)):
             self._ensure_dataset(of, output_shapes, output_keys, ds_i, **create_dataset_options)
-            paths = [self.paths[ds_i].replace(self.channel_keywords[0], output_key) for output_key in output_keys]
+            paths = [replace_last(self.paths[ds_i], self.channel_keywords[0], output_key) for output_key in output_keys]
             index_arrays = np.array_split(self.index_array[ds_i_i:(ds_i_i+ds_i_len)], ceil(ds_i_len/self.batch_size))
             print("predictions for dataset:", self.paths[ds_i])
             unsaved_batches = 0
@@ -612,14 +657,14 @@ class MultiChannelIterator(IndexArrayIterator):
         if self.datasetIO is None:
             self._open_datasetIO()
         if self.labels is not None:
-            label_path = self.paths[ds_i].replace(self.channel_keywords[0], '/labels')
+            label_path = replace_last(self.paths[ds_i], self.channel_keywords[0], '/labels')
             if label_path not in output_file:
                 output_file.create_dataset(label_path, data=np.asarray(self.labels[ds_i], dtype=np.string_))
-        dim_path = self.paths[ds_i].replace(self.channel_keywords[0], '/originalDimensions')
+        dim_path = replace_last(self.paths[ds_i], self.channel_keywords[0], '/originalDimensions')
         if dim_path not in output_file and dim_path in self.datasetIO:
             output_file.create_dataset(dim_path, data=self.datasetIO.get_dataset(dim_path))
         for ocidx, output_key in enumerate(output_keys):
-            ds_path = self.paths[ds_i].replace(self.channel_keywords[0], output_key)
+            ds_path = replace_last(self.paths[ds_i], self.channel_keywords[0], output_key)
             if ds_path not in output_file:
                 output_file.create_dataset(ds_path, shape=(self.ds_array[0][ds_i].shape[0],)+output_shapes[ocidx], dtype=self.dtype, **create_dataset_options) #, compression="gzip" # no compression for compatibility with java driver
 
@@ -639,19 +684,23 @@ class MultiChannelIterator(IndexArrayIterator):
         return void_masks
 
     def set_allowed_indexes(self, indexes):
-        super().set_allowed_indexes(indexes)
+        if indexes is None:
+            super().set_allowed_indexes(self.total_n) # reset allowed_indexes
+        else:
+            super().set_allowed_indexes(indexes)
         try: # reset void masks
             del self.void_masks
         except AttributeError:
             pass
+        self.group_proportion_init = False # reset group proportion init flag
 
     def __len__(self):
-        if self.void_mask_max_proportion>=0 and not hasattr(self, "void_masks") or self.void_mask_max_proportion<0 and hasattr(self, "group_proportion"):
+        if self.void_mask_max_proportion>=0 and not hasattr(self, "void_masks") or self.void_mask_max_proportion<0 and self.group_proportion is not None and not self.group_proportion_init:
             self._set_index_array() # redefines n in either cases
         return super().__len__()
 
     def _set_index_array(self):
-        if self.void_mask_max_proportion>=0: # in case there are too many void masks -> some are randomly removed
+        if self.void_mask_max_proportion>=0: # if void_mask_max_proportion is set. Use case: in case there are too many void masks -> some are randomly removed
             try:
                 void_masks = self.void_masks
             except AttributeError:
@@ -673,19 +722,19 @@ class MultiChannelIterator(IndexArrayIterator):
             else:  # only void or only not void
                 #print("void mask bins: ", bins)
                 index_a = self.allowed_indexes
-        elif hasattr(self, "group_proportion"): # generate a batch with user-defined proportion in each group
-            grp_prop = ensure_multiplicity(len(self.group_map_paths), self.group_proportion)
+        elif self.group_proportion is not None: # generate a batch with user-defined proportion in each group
             # pick indices for each group
             if self.allowed_indexes is None:
-                indices_per_group = [np.random.randint(low=self.grp_off[i], high=self.grp_len[i], size=int((self.grp_len[i] - self.grp_off[i])*grp_prop[i]+0.5) ) for i in range(len(self.group_map_paths))]
+                indices_per_group = [np.random.randint(low=self.grp_off[i], high=self.grp_len[i], size=int((self.grp_len[i] - self.grp_off[i])*self.group_proportion[i]+0.5) ) for i in range(len(self.group_map_paths))]
                 index_a = flatten_list(indices_per_group)
             else:
                 index_array = np.copy(self.allowed_indexes) # index within group
                 index_grp = self._get_grp_idx(index_array) # group index
                 allowed_indexes_per_group = [self.allowed_indexes[index_grp==i] for i in range(len(self.group_map_paths))]
-                indexes_per_group = [ pick_from_array(allowed_indexes_per_group[i], grp_prop[i]) for i in range(len(self.group_map_paths)) ]
+                indexes_per_group = [ pick_from_array(allowed_indexes_per_group[i], self.group_proportion[i]) for i in range(len(self.group_map_paths)) ]
                 index_a = np.concatenate(indexes_per_group)
             self.n = len(index_a)
+            self.group_proportion_init = True
         else:
             index_a = self.allowed_indexes
         if self.shuffle:
@@ -693,29 +742,77 @@ class MultiChannelIterator(IndexArrayIterator):
         else:
             self.index_array = np.copy(index_a)
 
-    def evaluate(self, model, metrics, progress_callback=None):
+    def evaluate(self, model, metrics, perform_data_augmentation=True, reset_allowed_indices=False, progress_callback=None, return_metadata=False):
+        """Evaluates model on this iterator.
+
+        Parameters
+        ----------
+        model : object
+            object with function predict(x) that return output numpy arrays
+        metrics : list
+            list of length equal to model output number.
+            Each element of the list can be a list of metrics, a single metric, or none.
+            A metric is a callable that takes two arguments: y_true and y_pred; or a string with value in: mse / mae / msem (mse within y_true!=0) /maem (mse within y_true!=0)
+        perform_data_augmentation : bool
+            whether data_augmentation should be performed or not during evaluation
+        progress_callback : callable
+            callable that take no argument called after each step.
+        return_metadata: bool
+            whether path, labels, indices should be returned or not (see below)
+
+        Returns
+        -------
+        tuple (values, path, labels, frames)
+            values: ndarray of rank 2.
+                each row corresponds to an image
+                first column is the global index of image
+                other columns correspond to the metric value, in the order of the metrics
+            path: ndarray of rank 1 type string
+                dataset path for each image
+            labels: ndarray of rank 1 type string
+                label of each image or none
+            frames: ndarray of rank 1 type Integer
+                frame of each image or none if labels are none
+        if return_metadata is False only values are returned
+
+        """
         output_number = sum(self.output_multiplicity.values()) if isinstance(self.output_multiplicity, dict) else len(self.output_channels)*self.output_multiplicity
         if len(metrics) != output_number:
             raise ValueError("metrics should be an array of length equal to output number ({})".format(output_number))
         shuffle = self.shuffle
         perform_aug = self.perform_data_augmentation
         self.shuffle=False
-        self.perform_data_augmentation=False
+        self.perform_data_augmentation=perform_data_augmentation
+        if reset_allowed_indices:
+            self.set_allowed_indexes(None)
         self.reset()
         self._set_index_array() # in case shuffle was true.
 
-        metrics = [l if isinstance(l, list) else ([] if l is None else [l]) for l in metrics]
+        metrics = [l if isinstance(l, (tuple, list)) else ([] if l is None else [l]) for l in metrics]
         # count metrics
+        ax = lambda tensor:tuple(range(1, tensor.shape))
         count = 0
-        for m in metrics:
-            count+=len(m)
-        values = np.zeros(shape=(len(self.allowed_indexes), count+2))
+        for l in metrics:
+            count+=len(l)
+            for i, m in enumerate(l):
+                if not callable(m):
+                    if m=="mse":
+                        m = lambda yt, yp:np.mean( (yt - yp)**2, axis=ax(yt) )
+                    elif m=="mae":
+                        m = lambda yt, yp:np.mean( np.abs(yt - yp), axis=ax(yt) )
+                    elif m=="msem" :
+                        m = lambda yt, yp:np.mean( (yt[yt!=0] - yp[yt!=0])**2, axis=ax(yt) )
+                    elif m=="maem" :
+                        m = lambda yt, yp:np.mean( np.abs(yt[yt!=0] - yp[yt!=0]), axis=ax(yt) )
+                    else:
+                        raise ValueError("Unsupported metric")
+        values = np.zeros(shape=(len(self.allowed_indexes), count+1))
         i=0
         for step in range(len(self)):
             x, y = self.next()
-            y_pred = model.predict(x=x, verbose=0)
+            y_pred = model.predict(x)
             n = y_pred[0].shape[0]
-            j=2
+            j=1
             for oi, ms in enumerate(metrics):
                 for m in ms:
                     res = m(y[oi], y_pred[oi])
@@ -731,18 +828,20 @@ class MultiChannelIterator(IndexArrayIterator):
 
         self.shuffle = shuffle
         self.perform_data_augmentation = perform_aug
-        # also return dataset dir , index and labels (if availables)
 
-        idx = np.copy(self.index_array)
-        ds_idx = self._get_ds_idx(idx)
-        values[:,0] = idx
-        values[:,1] = ds_idx
-        path = [self.paths[i] for i in ds_idx]
-        labels = [self.labels[i][j] for i,j in zip(ds_idx, idx)] if self.labels is not None else None
-        indices = [str(int(s[1]))+"-"+s[0].split('-')[1] for s in [l.split("_f") for l in labels]] #caveat: s[0].split('-')[1] is not the parent idx but the parentTrackHead idx, same in most case but ...
-        return values, path, labels, indices
+        values[:,0] = self.index_array
+        if return_metadata: # also return dataset path , labels and frame (if availables)
+            idx = np.copy(self.index_array)
+            ds_idx = self._get_ds_idx(idx)
+            path = [self.paths[i] for i in ds_idx]
+            labels = [self.labels[i][j] for i,j in zip(ds_idx, idx)] if self.labels is not None else None
+            frames = [str(int(s[1]))+"-"+s[0].split('-')[1] for s in [l.split("_f") for l in labels]] if labels is not None else None
+            return values, path, labels, frames
+        else:
+            return values
 
 # class util methods
+
 def _apply_multiplicity(batch, multiplicity):
     if batch is None:
         return
